@@ -4,4 +4,6 @@ from tmval.value import *
 from tmval.annuity import *
 from tmval.loan import *
 from tmval.bond import *
-from tmval.stock import *
\ No newline at end of file
+from tmval.stock import *
+from tmval.option import *
+from tmval.portfolio import *
\ No newline at end of file
