@@ -6,9 +6,11 @@ from collections import namedtuple
 import decimal
 import numpy as np
 from scipy.integrate import quad
+from scipy.optimize import newton
 
 from typing import (
     Callable,
+    Iterable,
     Union
 )
 
@@ -449,9 +451,9 @@ class Annuity(Payments):
                 sv = self.sbar_angln()
 
             else:
-                sv = self.amount * \
-                     ((1 + self.gr.interest_rate) ** self.term - 1) / \
-                     (self.gr.val(self.period) - 1)
+                i = self.gr.val(self.period) - 1
+                n = self.n_payments
+                sv = self.amount * ((1 + i) ** n - 1) / i
 
         elif self.aprog != 0 and self.mprog == 0:
 
@@ -934,7 +936,7 @@ def olb_p(
 
     else:
         ann = Annuity(
-            period=period,
+            period=min(period, term - t),
             term=term - t,
             gr=gr,
             amount=q
@@ -1024,11 +1026,25 @@ def get_perpetuity_pmt(
 
 
 def n_solver(
-        gr,
-        amount,
-        sv=None,
-        period=None,
-):
+        gr: Union[Accumulation, float, Rate],
+        amount: Union[float, int],
+        sv: Union[float, int] = None,
+        period: float = None,
+) -> float:
+    """
+    Given the present value, future value, growth rate, and payment interval, solves for the number of payments
+    in a level annuity.
+
+    :param gr: A growth rate object.
+    :type gr: Accumulation, float, Rate
+    :param amount: The present value of the annuity.
+    :type amount: float, int
+    :param sv: The future value of the annuity.
+    :type sv: float, int
+    :param period: The payment period.
+    :type period: float
+    :return: The number of periods.
+    """
     if sv:
         acc = standardize_acc(gr)
         i = acc.effective_rate(period)
@@ -1038,3 +1054,79 @@ def n_solver(
         n = None
 
     return n
+
+
+def isolve_multiple(
+        t1: float,
+        t2: float,
+        multiple: float,
+        period: float = 1,
+        x0=np.linspace(.001, 1, 100),
+        precision=5,
+        result_period: float = 1,
+) -> Rate:
+    """
+    Given two points in time, returns the growth rate needed for the investment to grow a specified number of multiples
+    between those two points in time. This function assumes that an investor is making regular contributions at a
+    specified interval into some kind of investment account. For example, we might want to know what interest rate
+    would be needed for the an investor who deposits 100 each year for the investment to grow 5 times between years
+    2 and 10.
+
+    :param t1: The point in time corresponding to the beginning of the relevant interval of growth.
+    :type t1: float
+    :param t2: The point in time corresponding to the end of the relevant interval of growth.
+    :type t2: float
+    :param multiple: The desired factor by which the investment will grow.
+    :type multiple: float
+    :param period: The time interval between payments.
+    :type period: float
+    :param x0: The initial guess used in Newton's method.
+    :type x0: float
+    :param precision: The desired precision of the result.
+    :type precision: float
+    :param result_period: The desired period for the returned growth rate.
+    :type result_period: float
+    :return: The required growth rate.
+    """
+
+    # Calculate the exponents used in the polynomial.
+
+    n1 = t1 / period
+
+    n2 = t2 / period
+
+    # Set the polynomial equal to zero in preparation for Newton's method.
+
+    def f(x):
+        return x ** n2 - multiple * (x ** n1) + (multiple - 1)
+
+    # Calculate the derivative, this improves the accuracy when used in Newton's method.
+
+    def fp(x):
+        return n2 * (x ** (n2 - 1)) - (multiple * n1) * (x ** (n1 - 1))
+
+    # Solve for the roots.
+
+    roots = newton(func=f, fprime=fp, x0=x0)
+
+    if isinstance(roots, Iterable):
+        sol = [round(x, precision) for x in roots]
+        acc = list(set(sol))
+
+    else:
+        acc = [roots]
+
+    acc = max(acc)
+
+    j = acc - 1
+
+    i = Rate(
+        rate=j,
+        pattern="Effective Interest",
+        interval=period
+    ).convert_rate(
+        pattern="Effective Interest",
+        interval=result_period
+    )
+
+    return i
