@@ -81,8 +81,17 @@ class Bond(Payments):
         self.k = k
 
         if [cgr, alpha].count(None) == 2:
-            c_args = None
-            self.alpha = None
+            # if bond is par and priced at par
+            if price == red == face and gr is not None:
+                alpha = standardize_acc(gr).interest_rate.convert_rate(
+                    pattern="Nominal Interest",
+                    freq=cfreq
+                ).rate
+                self.alpha = alpha
+                c_args = 1
+            else:
+                c_args = None
+                self.alpha = None
         else:
             c_args = len([cgr, alpha])
 
@@ -126,7 +135,14 @@ class Bond(Payments):
 
         n_missing = args.count(None)
 
-        if n_missing == 1:
+        if n_missing == 0:
+            self.red = red
+            self.n_coupons = self.get_n_coupons()
+            self.gr = standardize_acc(gr)
+            self.coupons = self.get_coupons()
+            self.price = price
+
+        elif n_missing == 1:
 
             if price is None:
                 self.n_coupons = self.get_n_coupons()
@@ -152,7 +168,6 @@ class Bond(Payments):
                         amounts += [afr[0]] * n
                     amounts += [red]
 
-                print(amounts)
                 times = [0.0] + self.get_coupon_times() + [term]
 
                 pmts = Payments(
@@ -249,14 +264,15 @@ class Bond(Payments):
 
                     j = self.gr.val(1 / self.cfreq) - 1
                     f = 1 - self.term / (1 / self.cfreq)
-                    print((1 + (1 - f) * j))
+
                     self.price = (self.red + self.fr) / (1 + (1 - f) * j) - f * self.fr
                 else:
                     self.price = self.clean(t=0)
         else:
             pass
 
-        self.append(amounts=[-self.price], times=[0])
+        self.amounts = [-self.price] + self.amounts
+        self.times = [0] + self.times
 
         if self.is_zero:
             pass
@@ -1347,3 +1363,51 @@ def forward_rates(term, bonds=None, yields=None, alpha=None):
         res.update({(t, t + term): f})
 
     return res
+
+
+def alm(
+        amounts,
+        bonds
+):
+    bd1_dict = bonds[0]
+    bd2_dict = bonds[1]
+    bd3_dict = bonds[-1]
+    alpha3 = bd3_dict['alpha']
+    cfreq3 = bd3_dict['cfreq']
+    fc = amounts[-1] / (1 + alpha3 / cfreq3)
+
+    bd3 = Bond(
+        face=fc,
+        red=fc,
+        alpha=alpha3,
+        cfreq=cfreq3,
+        term=bd3_dict['term'],
+        gr=bd1_dict['gr']
+    )
+
+    coupon3 = bd3.fr
+    alpha2 = bd2_dict['alpha']
+    cfreq2 = bd2_dict['cfreq']
+
+    fb = (amounts[-2] - coupon3) / (1 + alpha2 / cfreq2)
+
+    bd2 = Bond(
+        face=fb,
+        red=fb,
+        alpha=alpha2,
+        cfreq=cfreq2,
+        term=bd2_dict['term'],
+        gr=bd1_dict['gr']
+    )
+
+    gr1 = bd1_dict['gr']
+
+    fc = (amounts[-3] - bd3.fr - bd2.fr) / gr1.acc_func(.5)
+
+    bd1 = Bond(
+        face=fc,
+        red=fc,
+        gr=gr1,
+        term=bd3_dict['term']
+    )
+    return [bd1, bd2, bd3]
