@@ -10,6 +10,7 @@ import numpy as np
 from dateutil.relativedelta import relativedelta
 from inspect import signature
 from numpy import ndarray
+from scipy.integrate import quad
 from scipy.misc import derivative
 from scipy.optimize import newton
 from typing import Callable, Iterable, Tuple, Union
@@ -308,7 +309,7 @@ class Accumulation(Amount):
     """
     def __init__(
         self,
-        gr: Union[Callable, float, Rate]
+        gr: Union[Callable, float, int, Rate]
     ):
         super().__init__(
             gr=gr,
@@ -331,10 +332,11 @@ class Accumulation(Amount):
 
             return f
 
-        elif isinstance(self.gr, (float, Rate)):
+        elif isinstance(self.gr, (float, int, Rate)):
             return standardize_rate(self.gr).acc_func
         else:
-            raise Exception("Growth object must be a callable or Rate object.")
+
+            raise Exception("Growth object must be a callable, float, int, or Rate object.")
 
     def _validate_func(self):
         """
@@ -433,6 +435,10 @@ class Accumulation(Amount):
 
         return res
 
+    def dval(self, k, t1, t2):
+        k0 = self.discount_func(t=t1, fv=k)
+        return k0 * self.val(t2)
+
 
 def simple_solver(
     pv: float = None,
@@ -543,11 +549,11 @@ def actual_actual(
         return (end_dt - beg_dt).days
     else:
         years = relativedelta(end_dt, beg_dt).years
-        print(years)
+
         intermediate = dt.datetime(beg_dt.year + 1, end_dt.month, end_dt.day)
-        print(intermediate)
+
         days = (intermediate - beg_dt).days
-        print(days)
+
 
         t = years + days / 365
 
@@ -1039,8 +1045,6 @@ def rate_from_earned(iex: Tuple[float, float], iey: Tuple[float, float]) -> Rate
     y_amt = iey[0]
     y_t = iey[1]
 
-    print((1 / ((y_t - 1) - (x_t - 1)) - 1))
-
     gr = Rate((y_amt / x_amt) ** (1 / ((y_t - 1) - (x_t - 1))) - 1)
 
     return gr
@@ -1164,3 +1168,48 @@ def k_from_intdisc(
 
     return k
 
+
+def nominal_m_solver(im, dm) -> float:
+    """
+    Given a nominal interest and nominal discount rate with the same compounding frequency,
+    finds the compuonding frequency
+
+    :param im:
+    :type im:
+    :param dm:
+    :type dm:
+    :return:
+    :rtype:
+    """
+    return (im * dm) / (im - dm)
+
+
+def vaalern(s, r) -> float:
+    """
+    Given two compounding frequencies s, and r, finds n such that
+    .. math::
+       1-\frac{d^{(n)}}{n} = \frac{1 + \frac{i^{(s)}}{s}}{1 + \frac{i^{(r)}}{r}}
+
+    :param s:
+    :type s:
+    :param r:
+    :type r:
+    :return:
+    :rtype:
+    """
+
+    return (r * s) / (r - s)
+
+
+def acc_from_delta_t(delta_t: Callable) -> Callable:
+    def f(t):
+        return np.exp(quad(delta_t, 0, t)[0])
+
+    return f
+
+
+def amt_from_delta_t(delta_t: Callable) -> Callable:
+    def f(t, k):
+        return k * np.exp(quad(delta_t, 0, t)[0])
+
+    return f
\ No newline at end of file
