@@ -4,6 +4,8 @@ import numpy as np
 import warnings
 
 from numpy import ndarray
+from scipy.interpolate import approximate_taylor_polynomial
+from scipy.misc import derivative
 from scipy.optimize import newton
 
 from itertools import groupby
@@ -108,6 +110,48 @@ class Payments:
         self.amounts += amounts
         self.times += times
 
+    def paymentize(self, other, gr=None):
+        if gr is None:
+            gr = self.gr
+
+        if isinstance(other, Payments):
+            if (
+                isinstance(self.times, list) and
+                isinstance(self.amounts, list) and
+                isinstance(other.times, list) and
+                isinstance(other.amounts, list)
+            ):
+
+                times = self.times + other.times
+                amounts = self.amounts + other.amounts
+            else:
+                raise ValueError("Invalid object passed to argument 'other'.")
+
+        elif isinstance(other, list):
+            times = []
+            amounts = []
+            for x in other:
+                times += x.times
+                amounts += x.amounts
+        else:
+            raise ValueError("Invalid object passed to argument 'other'.")
+
+        grouped = Payments(
+            times=times,
+            amounts=amounts
+        ).group_payments()
+
+        times = [k for k in grouped.keys()]
+        amounts = [grouped[k] for k in grouped.keys()]
+
+        pmts = Payments(
+            times=times,
+            amounts=amounts,
+            gr=gr
+        )
+
+        return pmts
+
     def group_payments(self) -> dict:
         times = self.times.copy()
         amounts = self.amounts.copy()
@@ -122,11 +166,16 @@ class Payments:
 
         return payments_dict
 
-    def npv(self):
-        if self.gr is None:
-            raise Exception("Growth rate object not set.")
+    def npv(self, gr=None):
+        if gr is None:
+            if self.gr is None:
+                raise Exception("Growth rate object not set.")
+            else:
+                acc = self.gr
+        else:
+            acc = standardize_acc(gr=gr)
 
-        pv = sum([self.gr.discount_func(t=t, fv=fv) for t, fv in zip(self.times, self.amounts)])
+        pv = sum([acc.discount_func(t=t, fv=fv) for t, fv in zip(self.times, self.amounts)])
 
         return pv
 
@@ -164,14 +213,17 @@ class Payments:
         # if times are fractional, use Newton's method:
         else:
             def f(x):
-                return sum([payments_dict[k] * (x ** k) for k in payments_dict.keys()])
+                return sum([payments_dict[k] * (x ** - k) for k in payments_dict.keys()])
+
             roots = newton(func=f, x0=x0)
+
             if isinstance(roots, ndarray):
                 pass
             else:
                 roots = np.array(roots)
             reals = roots[np.isreal(roots)]
-            i_s = [(np.real(x) ** -1) - 1 for x in reals]
+
+            i_s = [(np.real(x)) - 1 for x in reals]
 
         return i_s
 
@@ -232,9 +284,16 @@ class Payments:
 
         # return bal
 
-    def eq_val(self, t: float) -> float:
+    def eq_val(self, t: float, gr=None) -> float:
+        if gr is None:
+            if self.gr is None:
+                raise Exception("Growth rate object not set.")
+            else:
+                acc = self.gr
+        else:
+            acc = standardize_acc(gr=gr)
 
-        b = sum([c * self.gr.val(t) / self.gr.val(tk) for c, tk in zip(self.amounts, self.times)])
+        b = sum([c * acc.val(t) / acc.val(tk) for c, tk in zip(self.amounts, self.times)])
 
         return b
 
@@ -341,6 +400,186 @@ class Payments:
 
         return jtw
 
+    def taylor(self, gr, x, degree):
+
+        def f(grs):
+            res = []
+            for i in grs:
+                res += [self.npv(gr=i)]
+            return res
+
+        #approx = approximate_taylor_polynomial(f, x=x, degree=degree, scale=gr)
+        return approximate_taylor_polynomial(f, x=x, degree=degree, scale=gr)
+
+    def tangent_line_approx(self, i0, i):
+
+        return self.npv(gr=i0) + derivative(self.npv, x0=i0, dx=1e-6) * (i - i0)
+
+    def taylor2(self, i0, i):
+
+        return self.tangent_line_approx(i0=i0, i=i) + \
+               derivative(self.npv, x0=i0, dx=1e-5, n=2) / 2 * (i - i0) ** 2
+
+    def relchg(self, i, i0=None, approx=False, excl_inv=True, degree=1):
+        if i0 is None:
+            if self.gr is not None:
+                i0 = self.gr
+            else:
+                raise ValueError("Growth rate object not set")
+        else:
+            i0 = standardize_acc(gr=i0)
+
+        if approx:
+            if i0.is_compound:
+                d1 = - self.modified_duration(i=i0.interest_rate, excl_inv=excl_inv) * (i - i0.interest_rate)
+                if degree == 1:
+                    res = d1
+                elif degree == 2:
+                    d2 = self.modified_convexity(i=i0.interest_rate, excl_inv=excl_inv) * ((i -i0.interest_rate) ** 2) / 2
+                    res = d1 + d2
+                else:
+                    raise ValueError("Relative change approximation is only supported for 1st and 2nd degrees.")
+            else:
+                raise Exception("Relative change approximation is unsupported for non-compound interest.")
+        else:
+            if excl_inv:
+                times = self.times.copy()
+                amounts = self.amounts.copy()
+                times.pop(0)
+                amounts.pop(0)
+                pmts = Payments(times=times, amounts=amounts, gr=self.gr)
+                res = (pmts.npv(gr=i) - pmts.npv(gr=i0)) / pmts.npv(gr=i0)
+            else:
+                res = (self.npv(gr=i) - self.npv(gr=i0)) / self.npv(gr=i0)
+
+        return res
+
+    def modified_duration(self, i, m=1, excl_inv=True):
+
+        if excl_inv:
+            if m != 1:
+                im = Rate(i).convert_rate(
+                    pattern="Nominal Interest",
+                    freq=m
+                )
+                return self.macaulay_duration() / (1 + im.rate / m)
+            else:
+                times = self.times.copy()
+                amounts = self.amounts.copy()
+                times.pop(0)
+                amounts.pop(0)
+                pmts = Payments(times=times, amounts=amounts, gr=self.gr)
+                return - derivative(pmts.npv, x0=i, dx=1e-6) / pmts.npv(gr=i)
+        else:
+
+            return - derivative(self.npv, x0=i, dx=1e-6) / self.npv(gr=i)
+
+    def macaulay_duration(self, gr=None, excl_inv=True):
+        if gr is None:
+            if self.gr is None:
+                raise Exception("Growth rate object not set.")
+            else:
+                acc = self.gr
+        else:
+            acc = standardize_acc(gr=gr)
+
+        if excl_inv:
+            times = self.times.copy()
+            amounts = self.amounts.copy()
+            times.pop(0)
+            amounts.pop(0)
+            pmts = Payments(times=times, amounts=amounts, gr=acc)
+            pv = pmts.npv()
+        else:
+            pv = self.npv(gr=gr)
+            times = self.times
+            amounts = self.amounts
+
+        md = sum([acc.discount_func(t=t, fv=fv) * t / pv for t, fv in zip(times, amounts)])
+
+        return md
+
+    def modified_convexity(self, i, m=1, excl_inv=True, dx=1e-5):
+        if excl_inv:
+            times = self.times.copy()
+            amounts = self.amounts.copy()
+            times.pop(0)
+            amounts.pop(0)
+            pmts = Payments(times=times, amounts=amounts, gr=self.gr)
+            p_double_prime = derivative(pmts.npv, x0=i, dx=dx, n=2)
+
+            return p_double_prime / pmts.npv(gr=i)
+
+    def macaulay_convexity(self, gr=None, excl_inv=True):
+
+        if gr is None:
+            if self.gr is None:
+                raise Exception("Growth rate object not set.")
+            else:
+                acc = self.gr
+        else:
+            acc = standardize_acc(gr=gr)
+
+        if excl_inv:
+            times = self.times.copy()
+            amounts = self.amounts.copy()
+            times.pop(0)
+            amounts.pop(0)
+            pmts = Payments(times=times, amounts=amounts, gr=acc)
+            pv = pmts.npv()
+        else:
+            pv = self.npv(gr=gr)
+            times = self.times
+            amounts = self.amounts
+
+        mc = sum([acc.discount_func(t=t, fv=fv) * t ** 2 / pv for t, fv in zip(times, amounts)])
+
+        return mc
+
+    def check_redington(self, precision=4):
+        if round(self.npv(), precision) == 0:
+            c1 = True
+        else:
+            c1 = False
+
+        if round(derivative(self.npv, x0=self.gr.interest_rate, dx=1e-6), precision) == 0.0:
+            c2 = True
+        else:
+            c2 = False
+
+        if round(derivative(self.npv, x0=self.gr.interest_rate, dx=1e-6, n=2), precision) >= 0.0:
+            c3 = True
+        else:
+            c3 = False
+
+        return c1 and c2 and c3
+
+    def effective_duration(self, i0, h, call=None, excl_inv=True):
+
+        if excl_inv:
+            times = self.times.copy()
+            amounts = self.amounts.copy()
+            times.pop(0)
+            amounts.pop(0)
+            pmts = Payments(times=times, amounts=amounts)
+        else:
+            pmts = self
+
+        if call is not None:
+            p1 = call
+        else:
+            p1 = pmts.npv(gr=i0 - h)
+
+        p2 = pmts.npv(gr=i0 + h)
+
+        p0 = pmts.npv(gr=i0)
+
+        mh = (p2 - p1) / (2 * h)
+
+        eh = - mh / p0
+
+        return eh
+
 
 def npv(
         payments: list,
@@ -685,4 +924,6 @@ def extract_flows(payments: Union[Payments, List[Payments]]) -> Payments:
         times=times
     )
 
-    return res
\ No newline at end of file
+    return res
+
+
